#ifndef HEADER_HH_
#define HEADER_HH_
#include <string>
#include <iostream>
#include <fstream>  
namespace Header
{
  enum SURVEY { S70=0, PM, PH, DUD };
  const std::string surveyNames[DUD+1] = {"70-cm survey", 
					  "Parkes Multibeam survey",
					  "Parkes High-latitude survey",
					  "Not a valid survey"};
  inline SURVEY idSurvey(std::string file)
  {
    std::ifstream fin;
    fin.open(file.c_str(), std::ios::binary);
    char *hdr = new char[5];
    fin.read( hdr, 5*sizeof(char) );
    fin.close();
    hdr[4] = '\0';
    unsigned char compstring_s70[5] = {182, 'm', 'm', 219, 0};
    unsigned char compstring_pm[4] = {'P', 'M', 'D'};
    compstring_pm[3] = '\0';
    if(strcmp((const char*)compstring_s70,hdr)==0) return S70;
    else if(strncmp((const char*)compstring_pm,hdr,3)==0) return PM;
    else return DUD;
  }

  enum ISMOVING { NOTGRID=0, POINT, MOVING, SCANNING };

  enum FIELDS 
    {CODE=0, VERSION, TAPENR, SCAN_CNTR, BLK_CNTR, 
     DATE, MJD, UT_STRT, LST_STRT, 
     COORD_TYPE, RA_STRT, DEC_STRT, GL_STRT, GB_STRT, 
     FEED_ANGLE, OBS_TIME, SCAN_RATE, SCAN_LEN, COMMENT, 
     NFILTER, CHANBW, NCHAN, FREQ_CHAN1, 
     SAMP_INT, SAMP_GRP, GRP_BLK, BLK_SEC, FDCNTRL, DATA_TYPE,
     UT, LST, RA, DEC, GL, GB, TEL_ZEN, TEL_AZ, 
     STATUS, ATTEN, TPOWER, NBLK_READ, SCAN_RATEL, SCAN_RATEB, 
     BYTE_GRP, NBEAM, IBEAM, PSRNAME, DATA_TWO, OBS_FILEN, 
     NBITS, DDM, NDDCH, MOVE, PNTERR, TREE, NSYS, 
     TELID, PANGLE, SMP_BSW, CAL_CYCLE, RESERVE
    };

  const int numKeywords = RESERVE + 1;


  class SurveySchema
  {
  public:
    SurveySchema(){};
    SurveySchema(SURVEY surveyType);
    SurveySchema(const SurveySchema& s)
    {
      for(int i=0;i<numKeywords;i++) startPosition[i]  = s.startPosition[i];
      for(int i=0;i<numKeywords;i++) fieldLength[i]    = s.fieldLength[i];
      for(int i=0;i<numKeywords;i++) fieldDimension[i] = s.fieldDimension[i];
      ant_x    = s.ant_x;
      ant_y    = s.ant_y;   
      ant_z    = s.ant_z;    
      frontend = s.frontend;
      fd_poln  = s.fd_poln;  
      fd_hand  = s.fd_hand; 
      fd_sang  = s.fd_sang; 
      fd_xyph  = s.fd_xyph; 
      backend  = s.backend; 
      beconfig = s.beconfig; 
      be_phase = s.be_phase;
      be_dcc   = s.be_dcc;
      tcycle   = s.tcycle;   
      trk_mode = s.trk_mode; 
      scanlen  = s.scanlen;  
      cal_mode = s.cal_mode; 
      cal_freq = s.cal_freq; 
      cal_dcyc = s.cal_dcyc; 
      cal_phs  = s.cal_phs;  
    };
    SurveySchema& operator= (const SurveySchema& s)
    {
      for(int i=0;i<numKeywords;i++) startPosition[i]  = s.startPosition[i];
      for(int i=0;i<numKeywords;i++) fieldLength[i]    = s.fieldLength[i];
      for(int i=0;i<numKeywords;i++) fieldDimension[i] = s.fieldDimension[i];
      ant_x    = s.ant_x;
      ant_y    = s.ant_y;   
      ant_z    = s.ant_z;    
      frontend = s.frontend;
      fd_poln  = s.fd_poln;  
      fd_hand  = s.fd_hand; 
      fd_sang  = s.fd_sang; 
      fd_xyph  = s.fd_xyph; 
      backend  = s.backend; 
      beconfig = s.beconfig; 
      be_phase = s.be_phase;
      be_dcc   = s.be_dcc;
      tcycle   = s.tcycle;   
      trk_mode = s.trk_mode; 
      scanlen  = s.scanlen;  
      cal_mode = s.cal_mode; 
      cal_freq = s.cal_freq; 
      cal_dcyc = s.cal_dcyc; 
      cal_phs  = s.cal_phs;  
    };
    virtual ~SurveySchema(){};
    int getStartPosition(int field){return startPosition[field];};
    int getFieldLength(int field){return fieldLength[field];};
    int getFieldDimension(int field){return fieldDimension[field];};
    template <class T> T readParameter(std::string headerString, int field,
				       int elementNumber = 0);
    class xBadSurvey{
    public:
      xBadSurvey(std::string s){surveyName=s;};
      virtual ~xBadSurvey(){};
      std::string getSurvey(){return surveyName;};
    private:
      std::string surveyName;
    };

    // accessor functions for PHU keyword values -- only get functions needed.
    double      getAntX(){return ant_x;};
    double      getAntY(){return ant_y;};
    double      getAntZ(){return ant_z;};
    std::string getFrontend(){return frontend;};
    std::string getFd_poln(){return fd_poln;};
    int         getFd_hand(){return fd_hand;};
    float       getFd_sang(){return fd_sang;};
    float       getFd_xyph(){return fd_xyph;};
    std::string getBackend(){return backend;};
    std::string getBeconfig(){return beconfig;};
    int         getBe_phase(){return be_phase;};
    int         getBe_dcc(){return be_dcc;};
    double      getTcycle(){return tcycle;};
    std::string getObs_mode(){return obs_mode;};
    std::string getTrk_mode(){return trk_mode;};
    float       getScanlen(){return scanlen;};
    std::string getCal_mode(){return cal_mode;};
    float       getCal_freq(){return cal_freq;};
    float       getCal_dcyc(){return cal_dcyc;};
    float       getCal_phs(){return cal_phs;};

 
  private:
    int startPosition[numKeywords];
    int fieldLength[numKeywords];
    int fieldDimension[numKeywords];

    // The following are PHU keywords that are survey-dependent.
    double      ant_x;
    double      ant_y;
    double      ant_z;
    std::string frontend;
    std::string fd_poln;
    int         fd_hand;
    float       fd_sang;
    float       fd_xyph;
    std::string backend;
    std::string beconfig;
    int         be_phase;
    int         be_dcc;
    double      tcycle;
    std::string obs_mode;
    std::string trk_mode;
    float       scanlen;
    std::string cal_mode;
    float       cal_freq;
    float       cal_dcyc;
    float       cal_phs;

    // This is the units for DAT_FREQ in the subint table
//     std::string freq_units;
    

  };

}


#endif /*HEADER_HH_*/
