#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <string>
#include "Block.hh"
#include "Header.hh"
#include "PSRFits.hh"
#include "Utils.hh"

using namespace Header;
using namespace Block;
using namespace FITS;
using std::string;

ISMOVING moveStatus(std::string file)
{
  std::ifstream fin;
  fin.open(file.c_str(), std::ios::binary);
  DataBlock block(idSurvey(file));
  block.readBlock(fin);
  fin.close();
  return ISMOVING(block.getKeyword<int>(MOVE));
}

int main(int argc, char * argv[])
{
  std::string fname;
  if(argc==2) fname = argv[1];
  else{
    std::cerr << "Usage: \"pvoconvert.x filename\"...\n";
    return 1;
  }

  SURVEY currentSurvey = idSurvey(fname);
  std::cerr << "File " << fname <<" : " << surveyNames[currentSurvey]<<"\n";
  if(currentSurvey == DUD){
    std::cerr << "Exiting...\n";
    return 1;
  }


//   ISMOVING status = moveStatus(fname);
//   if(status == NOTGRID){
//     std::cerr 
//       << "\aOops! This is not a gridded observation -- and so not a survey\n"
//       << "Exiting...\n";
//     return 1;
//   }
//   else if(status == SCANNING){
//     std::cerr 
//       << "\aOops! This is a scanning observation -- how did that happen?\n"
//       << "Exiting...\n";
//     return 1;
//   }
//   else if((status != MOVING)&&(status != POINT)){
//     std::cerr <<"\a\a\aOops! status is an unknown value (" 
// 	      << status << ")\nExiting...\n";
//     return 1;
//   }
  

  std::ifstream fin;
  fin.open(fname.c_str(), std::ios::binary);

  bool eof;
  DataBlock block(currentSurvey);
  block.readBlock(fin);
  
  std::string newRA  = block.getKeyword<std::string>(RA);
  std::string newDec = block.getKeyword<std::string>(DEC);
  int newBeam        = block.getKeyword<int>(IBEAM);
  std::string newUT  = block.getKeyword<std::string>(UT_STRT);
  std::string date   = block.getKeyword<std::string>(DATE);

  std::string newmove=block.getKeyword<std::string>(MOVE);

  int oldBeam;
  std::string oldRA,oldDec,oldUT,oldmove;

  PSRFits fitsfile;
  fitsfile.setUT(newUT);
  fitsfile.setDate(date);
  fitsfile.setBeam(newBeam);
  fitsfile.setFITSname(currentSurvey);
  if(fitsfile.fileExists()){
    std::cerr << "\aThe file "<< fitsfile.getFITSname() 
	      << " already exists.\nDon't want to overwrite, so exiting...\n";
    return 1;
  }
  if(fitsfile.openFile()) {
    std::cerr << "\aCannot open file "<< fitsfile.getFITSname()
	      << ".\nExiting...\n";
    return 1;
  }
  fitsfile.writePHU(block);
  
  do{

    if(block.telescopeOK()) {
      // if we aren't moving, write the subint data.
      if(fitsfile.writeSubint(block)){
	std::cerr << "Error. Exiting...\n";
	return 1;
      }
    }
    
    oldRA   = newRA;
    oldDec  = newDec;
    oldBeam = newBeam;
    oldUT   = newUT;
    oldmove = newmove;

    // We've done all we can for this block. Read a new one.
    eof = !block.readBlock(fin);

    if(!eof){ // if the file is still open
      
      // get info on the new block
      newBeam = block.getKeyword<int>(IBEAM);
      newRA   = block.getKeyword<std::string>(RA);
      newDec  = block.getKeyword<std::string>(DEC);
      newmove = block.getKeyword<std::string>(MOVE);
      newUT   = block.getKeyword<std::string>(UT);

      if((newBeam != oldBeam) || !block.telescopeOK()  ){
	//if we've moved, or it's a new beam, close the file.
	fitsfile.closeFile();
      }

      if( (fitsfile.getFilestate()==CLOSED) && block.telescopeOK() ){
	// if there is no file open, but we are not moving, 
	//  open a new file

	if(newmove!=oldmove){ 
	  // if this is true, then we were moving but now are not.
	  // we therefore need to set the new date and time.
	  fitsfile.setUT(newUT);
	  if(dmsToDec(oldUT) > dmsToDec(newUT))  date = addDay(date);
	  fitsfile.setDate(date);
	}
	fitsfile.setBeam(newBeam);  // always set the new beam number
	fitsfile.setFITSname(currentSurvey);
	fitsfile.openFile();
	fitsfile.writePHU(block);
      }

    }
	
	
  }while(!eof);
  
  fitsfile.closeFile();
  fin.close();

}
